import { CLOUD_SERVICE_PROVIDER_LIST, DEFAULT_CLOUD_CONTEXTS } from 'constant/others/other';
import useDrupalJsonApi from 'hooks/drupal_jsonapi';
import CloudContext from 'model/CloudContext';
import CloudServiceProvider from 'model/CloudServiceProvider';
import { createContext, useEffect, useState } from 'react';
import { getLocalStorageItem, setLocalStorageItem } from 'service/storage';

interface Message {
  text: string;
  color: 'primary' | 'secondary' | 'success' | 'danger' | 'warning' | 'info' | 'light' | 'dark';
}

interface AppState {
  cloudContextList: CloudContext[];
  messages: Message[];
  addMessage: (message: string) => void;
  addErrorMessage: (message: string) => void;
  removeMessage: (messageIndex: number) => void;
  isLoading: boolean;
  bypassAnimation: boolean;
}

export const useAppState = (): AppState => {
  const { getJsonData ,getEntityListAll } = useDrupalJsonApi();
  const [cloudContextList, setCloudContextList] = useState<CloudContext[]>([...DEFAULT_CLOUD_CONTEXTS]);
  const [messages, setMessages] = useState<Message[]>([]);
  const [bypassAnimation, setBypassAnimation] = useState(
    getLocalStorageItem('bypassAnimation', 'false') === 'true'
  );
  const [isLoading, setLoading] = useState(true);

  // Set cloud context list.
  useEffect(() => {
    const init = async () => {
      let newCloudContextList = [...DEFAULT_CLOUD_CONTEXTS];
      for (const cloudServiceProvider of CLOUD_SERVICE_PROVIDER_LIST) {
        try {
          const data = (await getEntityListAll('cloud_config', {}, cloudServiceProvider))
            .map((record: any) => {
              return {
                cloudServiceProvider: cloudServiceProvider as CloudServiceProvider,
                name: record.attributes.cloud_context,
                labelName: record.attributes.name
              };
            });
          newCloudContextList = [...newCloudContextList, ...data];
        } catch {
          // pass
        }
      }
      setCloudContextList(newCloudContextList);
      setLoading(false);
    };
    init();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  // Set JSON:API server URI.
  useEffect(() => {
    const init = async () => {
      try {
        const res = await fetch('/clouds/cloud_dashboard/config/jsonapi_server_uri');
        if (res.ok) {
          const temp = (await res.json()).uri;
          setLocalStorageItem('jsonapiServerUri', temp);
        }
      } catch (e) {
        console.warn('Warn: Could not get the server URL for the JSON:API.');
        console.warn(e);
      }
    };
    init();
  }, []);

  // Set the animation bypass flag.
  useEffect(() => {
    const init = async () => {
      const data = await getJsonData<{
        bypass_animation: 'true' | 'false'
      }>('/clouds/cloud_dashboard/config/bypass_animation');
      setLocalStorageItem('bypassAnimation',
        data.bypass_animation === 'true' ? 'true' : 'false'
      );
      setBypassAnimation(data.bypass_animation === 'true');
    }
    init();
  }, []);

  /**
   * Add message for display.
   * @param message The message.
   */
  const addMessage = (message: string) => {
    setMessages([...messages, {
      text: message,
      color: 'success',
    }]);
  }

  /**
   * Add error message for display.
   * @param message The message.
   */
  const addErrorMessage = (message: string) => {
    setMessages([...messages, {
      text: message,
      color: 'danger',
    }]);
  }

  /**
   * Remove message for display.
   * @param messageIndex The message index.
   */
  const removeMessage = (messageIndex: number) => {
    const newMessages = [...messages];
    newMessages.splice(messageIndex, 1);
    setMessages(newMessages);
  }

  return {
    cloudContextList,
    messages,
    addMessage,
    addErrorMessage,
    removeMessage,
    isLoading,
    bypassAnimation,
  };
};

export const AppContext = createContext<AppState>({
  cloudContextList: [],
  messages: [],
  addMessage: () => {},
  addErrorMessage: () => {},
  removeMessage: () => {},
  isLoading: true,
  bypassAnimation: true,
});
